--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r145
Project Date		: 20090516171532

File				: SKG.lua
Commit Author		: zhinjio
Commit Revision		: 124
Commit Date			: 20081226095548
************************************************************************
Description	:
	Main addon primary methods (oninit, onenable, ondisable) and
	associated goo.

TODO		:
	Still need usage message

	Not sure why my timer frame code is in here. Is there maybe a better
	place for it?
************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]
local SKG = LibStub( "AceAddon-3.0"):GetAddon( "SKG" )
local SKGL = LibStub( "AceLocale-3.0"):GetLocale( "SKG" )

local addon = SKG

-- Show Config options table
-- Needs to be defined here since Locales must be loaded, and the functions
-- referred to must pass dewdrop arg checks (as such, they must exist)
fubarOptionsTable = {
	name = "SuicideKingsGeo",
	desc = "SuicideKingsGeo",
	handler = SKG,
	type = 'group',
	args = {
		["GenOpts"] = {
			name = SKGL["SKG_LBL_OF_TITLE_TEXT"],
			desc = SKGL["SKG_LBL_OF_TITLE_TEXT"],
			type = 'group',
			args = {
				["raidonly"] = {
					name = SKGL["SKG_LBL_OF_RAIDONLYCB_TEXT"],
					desc = SKGL["SKG_LBL_OF_RAIDONLYCB_TEXT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.raidonly
					end,
					order = 100,
					set = SKG_OF_RaidOnlyToggle,
				},
				["completelist"] = {
					name = SKGL["SKG_LBL_OF_SHOWALLCB_TEXT"],
					desc = SKGL["SKG_LBL_OF_SHOWALLCB_TEXT"],
					type = 'toggle',
					get = function() 
						return not SKG.db.profile.Options.General.raidonly
					end,
					order = 110,
					set = SKG_OF_RaidOnlyToggle,
				},
				["displayclasscolors"] = {
					name = SKGL["SKG_LBL_OF_SHOWCLASSCOLORSCB_TEXT"],
					desc = SKGL["SKG_LBL_OF_SHOWCLASSCOLORSCB_TEXT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.classColors
					end,
					order = 120,
					set = SKG_OF_ShowClassColorsClicked,
				},
				["spacer1"] = {
					type = "header",
					order = 130,
				},
				["autoopen"] = {
					name = SKGL["SKG_LBL_OF_AUTOOPENCB_TEXT"],
					desc = SKGL["SKG_OF_AUTOOPENCB_TT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.autoopen
					end,
					order = 140,
					set = SKG_OF_AutoOpenClicked,
				},
				["spacer2"] = {
					type = "header",
					order = 150,
				},
				["debug"] = {
					name = SKGL["SKG_LBL_OF_DEBUGCB_TEXT"],
					desc = SKGL["SKG_OF_DEBUGCB_TT"],
					type = 'toggle',
					get = function() 
						return SKG.db.profile.Options.General.debug
					end,
					order = 160,
					set = SKG_OF_DebugClicked,
				}
			}
		}
	}
}

-- Check to see if our SV database is up to date with current specs, etc.
function SKG:upgradeDB()
	local olddb = addon.db.profile
	if not olddb.Options.General.dbVersion then
		-- file existed before I added in db versioning. Do the right thing.
		olddb.Options.General.dbVersion = "1.0"
		SKG_Print( string.format( SKGL["Database upgraded to %s"], "1.0" ) )
	end
	--[[
	if ( olddb.Options.General.dbVersion == "1.0" ) then
		olddb.data.rested =0
		olddb.options.display.dbVersion = "1.1"
		addon.Print( string.format( L["Database upgraded to %s"], "1.1" ) )
	end
	]]--
end

-- on init
function SKG:OnInitialize()
	-- Frame setup if needed
end

function SKG:OnEnable()
    -- Called when the addon is loaded
    self.db = LibStub( "AceDB-3.0" ):New( "SKG_DB", nil, "Default" )

	if not ( self.db.profile.firstLoad ) then
		sdbp = self.db.profile
		sdbp.firstLoad = true
		sdbp.Players = {}
		sdbp.Alts = {}
		sdbp.RollLists = {}
		sdbp.LastMod = {}
		sdbp.Log = {
			["SKG"] = {},
			["Loot"] = {},
		}
		sdbp.Snapshot = {}
		sdbp.SnapshotLM = {}
		sdbp.Options = {
			General = {
				debug = false,
				autoopen = false,
				buttonTexture = 1,
				classColors = true,
				raidonly = false,
				dbVersion = "1.0",
			},
			Security = {
				Policies = {},
			},
			Bid = {
				ArmorFilter = 0,
			},
			List = {},
		}
	else
		-- check if we need to upgrade the db
		addon.upgradeDB()
	end

	-- Event listeners (handlers are in SKG_Events.lua)
	SKG:RegisterEvent( "ADDON_LOADED" )
	SKG:RegisterEvent( "LOOT_OPENED" )
	SKG:RegisterEvent( "RAID_ROSTER_UPDATE" )
	SKG:RegisterEvent( "CHAT_MSG_WHISPER" )
	SKG:RegisterEvent( "UNIT_SPELLCAST_SENT" )
	-- Other events to worry about
	-- PARTY_LOOT_METHOD_CHANGED
	SKG:RegisterEvent( "GUILD_ROSTER_UPDATE" )

	SKG_INIT_GUILD = 1

	-- Comm registration (handlers are in SKG_Comm.lua)
	SKG:RegisterComm( SKG_CHANNEL_PREFIX )

	-- Called when the addon is enabled
	local major, minor = SKG_getVersionComponents()
	SKG_Print( string.format( SKGL["SKG_MSG_ADDON_LOADED"], major, minor ) )
end

function SKG:OnDisable()
    -- Called when the addon is disabled
	SKG_ENABLED = false
	-- do my best to make the addon disappear from view
	SKG_Frame:Hide()
	-- get rid of the chat command
	SKG:UnregisterChatCommand( "skg" )
end

function SKG:Disable()
    -- Called when the addon is disabled
	SKG_ENABLED = false
	-- do my best to make the addon disappear from view
	SKG_Frame:Hide()
	-- get rid of the chat command
	SKG:UnregisterChatCommand( "skg" )
end

-- Register our chat command, and set its function
SKG:RegisterChatCommand("skg", "OnSlashCommand")

-- And now do the dirty work
function SKG:OnSlashCommand( text )
	if ( SKG_ENABLED == true ) then
		local db = SKG.db.profile.Options.General
		if ( text == "" ) then
			if ( SKG_Frame:IsVisible() ) then
				PlaySound("igMainMenuClose");
				SKG_Frame:Hide()
			else
				PlaySound("igMainMenuOpen");
				SKG_Frame:Show()
			end
		elseif ( text == "reset" ) then
			db.xPos = 100
			db.yPos = -100
			SKG_Frame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 100, -100 )
			SKG_Frame:Show()
		elseif ( text == "checklists" ) then
			-- These commands should only work if you're a syncer (or above) 
			if ( SKG_SECURITY_LEVEL ~= "U" ) then
				SKG_Print( SKGL["SKG_MSG_DIAG"] )
				-- perform a diagnostic check on lists to see if they have corruption
				-- Check Players list
				local rc = SKG_checkList( "Players" )
				SKG_Print( SKG_ListError( "Players", rc ) )
				-- Check Alts list
				rc = SKG_checkList( "Alts" )
				SKG_Print( SKG_ListError( "Alts", rc ) )
				-- loop through the rolling lists
				local db = SKG.db.profile.RollLists
				for k, v in pairs( db ) do
					rc = SKG_checkList( k )
					SKG_Print( SKG_ListError( k, rc ) )
				end
				SKG_Print( SKGL["SKG_MSG_DIAGCOMPLETE"] )
			else
				SKG_Print( SKGL["SKG_ERROR_NOTLEET"] )
			end
		elseif ( text == "repairlists" ) then
			-- make an attempt to get rid of bad entries in lists
			if ( SKG_SECURITY_LEVEL ~= "U" ) then
				SKG_Print( SKGL["SKG_MSG_LISTREPAIR"] )
				SKG_RepairLists()
				SKG_Print( SKGL["SKG_MSG_REPAIRCOMPLETE"] )
			else
				SKG_Print( SKGL["SKG_ERROR_NOTLEET"] )
			end
		else
			SKG_Print( SKGL["SKG Usage:"] )
			SKG_Print( "  " .. SKGL["/skg - open SKG GUI"] )
			SKG_Print( "  " .. SKGL["/skg reset - reset SKG GUI position"] )
			SKG_Print( "  " .. SKGL["/skg checklists - check lists for corruption"] )
			SKG_Print( "  " .. SKGL["/skg repairlists - attempt to repair list corruption"] )
		end
	end
end

local LDB = LibStub( "LibDataBroker-1.1", true )
local launcher = LDB:NewDataObject( "SKG", {
	type = "launcher",
	icon = "Interface\\Icons\\INV_Elemental_Primal_Nether",
	OnClick = function(clickedframe, button)
		if ( button == "RightButton" ) then
			SKG_ShowConfig( clickedframe )
		else
			SKG_Toggle()
		end
	end,
	OnTooltipShow = function( tooltip )
		tooltip = tooltip or GameTooltip
		tooltip:AddLine( SKGL["SKG_LBL_OF_TITLE_TEXT"], 1, 1, 1 )
		tooltip:AddLine( " " )
		local db = SKG.db.profile.Options.General
		local text1 = SKGL["SKG_LBL_OF_RAIDONLYCB_TEXT"]
		local text2 = ""
		if ( db.raidonly ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		local text1 = SKGL["SKG_LBL_OF_SHOWCLASSCOLORSCB_TEXT"]
		if ( db.classColors ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		local text1 = SKGL["SKG_LBL_OF_AUTOOPENCB_TEXT"]
		if ( db.autoopen ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		local text1 = SKGL["SKG_LBL_OF_DEBUGCB_TEXT"]
		if ( db.debug ) then text2 = SKGL["SKG__TRUE"] else text2 = SKGL["SKG__FALSE"] end
		tooltip:AddLine( text1 .. " : " .. text2 )
		tooltip:AddLine( " " )
		tooltip:AddLine( SKGL["SKG_LBL_LEFTCLICK"], 1, 1, 1 )
		tooltip:AddLine( SKGL["SKG_LBL_RIGHTCLICK"], 1, 1, 1 )
	end,		

} )

-- Frames used for resolving class conflicts during an import.

SKG.importClassSelections = {
	type = "group",
	handler = SKG,
	args = { },
}

-- Class chooser frame
	SKG_ChooseClassFrame = CreateFrame( "Frame", "SKG_ChooseClassFrame", UIParent)
	SKG_ChooseClassFrame:SetHeight( 125 )
	SKG_ChooseClassFrame:SetWidth( 256 )
	SKG_ChooseClassFrame:SetPoint( "CENTER", UIParent, "CENTER", 0, 0 )
	SKG_ChooseClassFrame:EnableMouse( true )
	SKG_ChooseClassFrame:SetMovable( false )
	SKG_ChooseClassFrame:SetFrameLevel( 1 )
	SKG_ChooseClassFrame:SetFrameStrata( "DIALOG" )
	SKG_ChooseClassFrame:Hide()

	local SKG_CCF_Texture = SKG_ChooseClassFrame:CreateTexture( "SKG_CCF_Texture", "ARTWORK" )
		SKG_CCF_Texture:SetTexture( [[Interface\Addons\SKG\BLP\GenericDialog]] )
		SKG_CCF_Texture:SetAllPoints( SKG_ChooseClassFrame )

	local SKG_CCF_TitleText = SKG_ChooseClassFrame:CreateFontString( "SKG_CCF_TitleText", "OVERLAY", "GameFontHighlight" )
		SKG_CCF_TitleText:SetText( SKGL["SKG_LBL_CCF_TITLE_TEXT"] )
		SKG_CCF_TitleText:SetPoint( "TOP", SKG_ChooseClassFrame, "TOP", 0, -15 )
		SKG_CCF_TitleText:SetJustifyH( "CENTER" )
		SKG_CCF_TitleText:SetHeight( 18 )
		SKG_CCF_TitleText:SetWidth( 200 )

	local SKG_CCF_NameText = SKG_ChooseClassFrame:CreateFontString( "SKG_CCF_NameText", "OVERLAY", "GameFontNormal" )
		SKG_CCF_NameText:SetText( "" )
		SKG_CCF_NameText:SetPoint( "TOP", SKG_ChooseClassFrame, "TOP", 0, -40 )
		SKG_CCF_NameText:SetJustifyH( "CENTER" )
		SKG_CCF_NameText:SetHeight( 18 )
		SKG_CCF_NameText:SetWidth( 200 )

	local SKG_CCF_ClassChooser = SKG_CreateButton( "SKG_CCF_ClassChooser", SKG_ChooseClassFrame, 21, 200,
		"TOP", SKG_CCF_NameText, "BOTTOM", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_CCF_SELECTCLASS_TEXT"], "CENTER", "SKG_CCF_SELECTCLASS_TT" )

		-- onclick method
		SKG_CCF_ClassChooser:SetScript( "OnClick",
			function()
				Dewdrop:Register( this,
					'children', SKG.importClassSelections,
					'dontHook', true,
					'point', "TOP",
					'relativePoint', "BOTTOM"
				)
				this:SetScript( "OnClick",
					function()
						if Dewdrop:IsOpen( this ) then
							Dewdrop:Close()
						else
							Dewdrop:Open( this )
						end
					end
				)
				this:GetScript( "OnClick" )()
			end
		)

	local SKG_CCF_OK =  SKG_CreateButton( "SKG_CCF_OK", SKG_ChooseClassFrame, 21, 120,
		"TOP", SKG_CCF_ClassChooser, "BOTTOM", 0, -5, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG__SUBMIT"], "CENTER", "SKG__SUBMIT" )

		SKG_CCF_OK:SetScript( "OnClick", function ()
			-- Validate our data
			if SKG.importSelectedClass == 0 then
				SKG_Print( SKGL["SKG_ERROR_NOCLASSSELECTED"] )
			else
				SKG_ChooseClassFrame:Hide()
				SKG.Import.Players[SKG.importCurrPID].class = strupper( SKG_ClassNames[SKG.importSelectedClass] )
				SKG.importSelectedClass = 0
				SKG_CCF_ClassChooser:SetText( SKGL["SKG_LBL_CCF_SELECTCLASS_TEXT"] )

				-- and trigger the next one
				SKG_ImportTimerFrame:Show()
			end
		end )

-- init code for above frame
SKG.importClassSelections.args = {}
SKG.importSelectedClass = 0
for i=1, #SKG_ClassNames do
	local t = {
		type = "execute",
		name = SKG_ClassNames[i],
		desc = SKG_ClassNames[i],
		func = function ( val )
			SKG.importSelectedClass = val
			SKG_CCF_ClassChooser:SetText( SKGL["SKG_LBL_CCF_SELECTCLASS_TEXT"] .. ": " .. SKG_ClassNames[val] )
			Dewdrop:Close()
		end,
		passValue = i
	}
	tinsert( SKG.importClassSelections.args, t )
end
SKG_CCF_ClassChooser:Enable()

--This is a timer frame

SKG_ImportTimerFrame = CreateFrame("Frame",SKG_ImportTimerFrame,UIParent)
	SKG_ImportTimerFrame.t = 0
	SKG_ImportTimerFrame:SetScript("OnUpdate", function(self,elapsed)
		SKG_ImportTimerFrame.t = SKG_ImportTimerFrame.t + elapsed;
		if ( SKG_ImportTimerFrame.t > 1 ) then 
			SKG_ImportTimerFrame:Hide()
			SKG_ImportTimerFrame.t = 0
			-- Look through the SKG.Import.Players structure to find an
			-- entry that has no "Class" associated with it. If you find
			-- one, kick off the dialog to resolve that, and hide
			local pList = SKG.Import.Players
			local found = false
			local pName = ""
			local pClass = ""
			for PID, pInfo in pairs( pList ) do
				pClass = pInfo.class
				if ( pClass == "" ) then
					pName = pInfo.name
					SKG.importCurrPID = PID
					found = true
					break
				end
			end
			if ( found == false ) then
				SKG_Print( SKGL["SKG_MSG_IMPORT_ENDPHASE2"] )
				-- execute on phase 3
				-- insert all entries in the imported players list into our
				-- main list
				local db = SKG.db.profile
				local pList = db.Players
				for PID, pInfo in pairs( SKG.Import.Players ) do
					if ( pList[PID] ) then
						-- it exists, no touchie
					else
						pList[PID] = pInfo
					end
				end
				local rList = db.RollLists
				for listName, listData in pairs( SKG.Import.RollLists ) do
					rList[listName] = listData
					-- clean up lastmod and make a snapshot
					SKG_CleanUpImport( listName )
				end
				-- force all relevant stuff to recalculate
				SKG_UpdateLists()
				SKG_ScrollFrame.selected = 0
				SKG_UpdateSPlayers()
				SKG_UpdateCurrentList()
				SKG_UpdateMainsChooser()
				SKG_UpdateToonChooser()

				if ( SKG.inRaid == true ) then
					SKG_UpdateRaiders()
				end

				SKG.Import = {}
				SKG.Import.Players = {}
				SKG.Import.RollLists = {}
				SKG.importClassSelections = {}
				SKG.importSelectedClass = 0
				SKG.importCurrPID = ""

				SKG_Print( SKGL["SKG_MSG_IMPORT_ENDPHASE3"] )
			else
				SKG_CCF_NameText:SetText( pName )
				SKG_ChooseClassFrame:Show()
			end
		end
	end )
	SKG_ImportTimerFrame:Hide()
--[[
************************************************************************
CHANGELOG:

Date : 10/26/08
	Moving RegisterEvent calls to OnInit
	Moving RegisterComm call to OnInit
	Defined fubarOptionsTable to be global

Date : 10/21/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer.
************************************************************************
]]--